#include <TFile.h>
#include <TTree.h>
#include <vector>
#include <algorithm>
#include <string>
#include <iostream>
#include <sstream>

using namespace std;

double rand_max = RAND_MAX; 
string ffname("C:/Work/RootTestCode/RootFiles/CintTestFile.root");

bool ReadTree(const string filename)
{
	TFile * file  = TFile::Open(filename.c_str(),"READ");	     
	if (!file) { return false; }
	cout<< "\n\nReading data..." << endl;

	TTree * test_tree; 
	file->GetObject("test_tree",test_tree);
	int i;
	double x;
	vector<string> *vs = 0;
	vector<double> *vd = 0;
	test_tree->SetBranchAddress("IntBranch",&i);
	test_tree->SetBranchAddress("DoubleBranch",&x);
	test_tree->SetBranchAddress("StringVector",&vs);
	test_tree->SetBranchAddress("DoubleVector",&vd);

	int ntries = test_tree->GetEntries();
	for(int e=0;e<ntries ;e++)
	{		
		int flag = test_tree->LoadTree(e);
		if (flag != e) {
			cout << "error: entry = " << e << ", LoadTree(entry) = " << flag << endl;
			return false;
		}
		//this crashes the program
		test_tree->GetEntry(e);	
		cout<<"Read Entry ["<<e<<"/"<<ntries<<"]:Int branch = "<<i<<", DblBranch = "<<x<<", Vs Data=[";
		for (vector<string>::iterator j = vs->begin(); j != vs->end(); ++j )
		{
			cout<< *j << ",";
		} 
		cout<<"], vd data=[";
		for (vector<double>::iterator k = vd->begin(); k != vd->end(); ++k )
		{
			cout<< *k << ",";
		} 
		cout<<"]"<<endl;
	}	
	// Since we passed the address of a local variable we need
	// to remove it.
	test_tree->ResetBranchAddresses();
	file->Close();
}

bool WriteTree(const string filename)
{
	TFile file(filename.c_str(), "RECREATE");//"path/tilename.root"
	if (file.IsZombie()) { return false; }
	cout<< "\n\nWriting data..." << endl;

	int i;
	double x;
	vector<string> vs(4);
	vector<double> vd(4);

	TTree * test_tree = new TTree("test_tree", "This is the test tree title");
	test_tree->Branch("IntBranch",&i);
	test_tree->Branch("DoubleBranch",&x);
	test_tree->Branch("StringVector",&vs);
	test_tree->Branch("DoubleVector",&vd);
	int ntries= 5;	
	for (i=0; i < ntries; ++i) 
	{
		x = rand()/rand_max;		

		cout<<"Write Entry ["<<i<<"/"<<ntries<<"]:Int branch = "<<i<<", DblBranch = "<<x<<", Vs Data=[";

		for(int j=0;j<vs.capacity();j++)
		{	
			stringstream stmp;
			stmp << "String data" << i << j;
			string tmp = stmp.str();
			cout<<tmp<<",";
			vs[j] = tmp;
		}
		cout<<"], vd data=[";
		for(int k=0;k<vd.capacity();k++)
		{			
			double dbl = rand()/rand_max;
			cout<<dbl<<",";
			vd[k] = dbl;
		}
		cout<<"]"<<endl;
		test_tree->Fill();
	}
	file.Write();
	file.Close();
	return true;
}

void read_write_test(const string filename = ffname)
{
	WriteTree(filename);
	ReadTree(filename);
}